/*!
*	\addtogroup Modulo2
*	\{
*/
/*!
*	\addtogroup TestModulo2 Specifiche per il Test
*	\{
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "LPC_Include.h"


static int handle = -1;

void TestOpen(void);	//!< Funzione di test per la OpenDatabaseFile()
void TestDBFInfo(void);	//!< Funzione di test per la GetDBFInfo()
void TestFieldInfo(void);	//!< Funzione di test per la GetFieldInfo()
void TestDBFHandle(void);	//!< Funzione di test per la IsDBFHandle()
void TestClose(void);	//!< Funzione di test per la CloseDatabaseFile()
void TestMisto(void);	//!< Funzione di test misto per tutte le funzioni
void Result(char *Func);	//!< Funzione di calcolo e stampa risultati

// Struttura di test misto
typedef struct {
	char Nome[15];
	int handle;
} DBF_TEST_t;
static DBF_TEST_t DBFTest[MAX_DBF_OPEN+1];

// Vettori per gli id dei test, il check e i pesi relativi
#define MAX_TEST 100
typedef struct {
	int testid;
	int weight;
	char desc[256];
	int passed;
} TEST_t;
		
static int numtest;
static TEST_t test[MAX_TEST];


/*!
*	\brief Test per le funzioni di interfaccia del Modulo 2
*
* Programma di test per il Modulo 2
*
* Ogni test e' identificato da un numero cosi' composto:\n
* \p xyynn \n
*  x	Id del modulo\n
* yy	Id della funzione\n
* nn	Id del test\n
*
* Ogni test ha anche un peso che viene indicato nella stampa dei risultati.
* I test di funzionalit hanno i pesi piu' elevati.
*
* Tramite menu possono essere richiamate sia le singole funzioni di test
* che un test completo con un punteggio finale.
*
* Vengono effettuati i seguenti controlli: 
* - Apertura (OpenDatabaseFile()): 
*   - Nome del DBF errato (NULL o caratteri non validi o inizio non valido)
*	- DBF non esistente
*   - Verifica funzionalita'
*	- Verifica nuova apertura DBF aperto
*
* - Informazioni DB (GetDBFInfo()):
*   - Handle del DBF errato
*   - Variabile parametro NULL
*   - Verifica funzionalita'
* 
* - Informazioni Campi (GetFieldInfo()):
*   - Handle del DBF errato
*   - Numero campi non valido
*   - FieldData non valido
*   - Campo non valido (Nome)
*   - Verifica funzionalita'
* 
* - Verifica handle (IsDBFHandle()):
*   - Verifica funzionalita' (handle valido o no)
*
* - Chiusura (CloseDatabaseFile()):
*   - Handle del DBF errato
*   - Verifica funzionalita'
****************************************************/



int main(int argc, char **argv)
{
	char scelta[10];
	int sel;

	printf("Programma di Test per il modulo 2\n\n");
	for(;;)
	{
		printf("\t1. Test funzione OpenDatabase\n");
		printf("\t2. Test funzione GetDBFInfo\n");
		printf("\t3. Test funzione GetFieldInfo\n");
		printf("\t4. Test funzione CloseDatabase\n");
		printf("\t5. Test funzione IsDBFHandle\n");
		printf("\t6. Test Misto di tutte le funzioni\n");
		printf("\t7. Test completo e valutazione\n");
		printf("\n\t0. Uscita\n");
		printf("\n\nScelta: ");

		(void) fgets(scelta, sizeof(scelta), stdin);
		sel = atoi(scelta);
		Result(NULL);
		switch(sel)
		{
		case 0:
			printf("Bye!\n\n");
			return(0);
		case 1:
			TestOpen();
			Result("OPEN");	
			break;
		case 2:
			TestDBFInfo();
			Result("DBFINFO");
			break;
		case 3:
			TestFieldInfo();
			Result("FIELDINFO");
			break;
		case 4:
			TestDBFHandle();
			Result("ISDBFHANDLE");
			break;
		case 5:
			TestClose();
			Result("CLOSE");
			break;
		case 6:
			TestMisto();
			Result("MISTO");
			break;
		case 7:
			TestOpen();
			TestDBFInfo();
			TestFieldInfo();
			TestClose();
			TestMisto();
			Result("FINALE");
			break;
		default:
			printf("Scelta errata...\n");
			break;;
		}
	}
}
void TestOpen()
{
	int ret= 0;

	printf("Test OPEN\n");

// Test sul nome
	// null
	test[numtest].testid = 20101;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "OPEN: nome NULL");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = OpenDatabaseFile(NULL);
	printf("Ritorno della open: [%d]\n", ret);
	if (ret < 0)
		test[numtest].passed = 1;
	numtest++;

	// Non valido
	test[numtest].testid = 20102;
	test[numtest].weight = 1;
	strcpy(test[numtest].desc, "OPEN: nome non valido");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = OpenDatabaseFile("PIppo-1");
	printf("Ritorno della open: [%d]\n", ret);
	if (ret < 0)
		test[numtest].passed = 1;
	numtest++;

// File non esistente
	test[numtest].testid = 20103;
	strcpy(test[numtest].desc, "OPEN: DBF non esistente");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	test[numtest].weight = 2;
	ret = OpenDatabaseFile("HKYDF");
	printf("Ritorno della open: [%d]\n", ret);
	if (ret < 0)
		test[numtest].passed = 1;
	numtest++;

// Funzionalita'
	// rimuovo lock
	remove("Prova.$$$");
	// Apertura
	test[numtest].testid = 20104;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "OPEN: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	handle = OpenDatabaseFile("Prova");
	printf("Ritorno della open: [%d]\n", handle);
	if (handle >= 0)
		test[numtest].passed = 1;
	numtest++;

	// Gia' aperto
	test[numtest].testid = 20105;
	test[numtest].weight = 4;
	strcpy(test[numtest].desc, "OPEN: file gia' aperto");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = OpenDatabaseFile("Prova");
	printf("Ritorno della open: [%d]\n", ret);
	if (ret < 0)
		test[numtest].passed = 1;
	numtest++;


}

void TestDBFInfo()
{
	int ret= 0, handle0;
	DBF_INFO_t DBFInfo;

	printf("Test DBFInfo. Da eseguire dopo i test sulla Open\n");

// Test sull'handle
	test[numtest].testid = 20201;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "DBFInfo: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetDBFInfo(400, &DBFInfo);
	printf("Ritorno della DBFInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Test parametro errato
	test[numtest].testid = 20202;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "DBFInfo: parametro NULL");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetDBFInfo(handle, NULL);
	printf("Ritorno della DBFInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 20203;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "DBFInfo: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	DBFInfo.NumFields = 0;
	ret =  GetDBFInfo(handle, &DBFInfo);
	printf("Ritorno della DBFInfo: [%d]\n", ret);
	if (ret == LPC_OK)
	{
		printf("Database modificato il [%s]\n", ctime(&DBFInfo.LastUpdate));
		if (DBFInfo.NumFields == 6)
			test[numtest].passed = 1;
	}
	numtest++;
 
// Verifica se legge dall'header
	test[numtest].testid = 20204;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "DBFInfo: verifica lettura da header");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	handle0 = OpenDatabaseFile("Prova0");
	if (handle0 >= 0)
	{
		ret =  GetDBFInfo(handle0, &DBFInfo);
		printf("Ritorno della DBFInfo: [%d]\n", ret);
		if (ret == LPC_OK)
		{
			if (!strcmp(DBFInfo.Name, "SgNaRf"))
				test[numtest].passed = 1;
		}
	}
	CloseDatabaseFile(handle0);
	numtest++;

}


void TestFieldInfo()
{
	int ret = 0;
	int NumFields = 0;
	FIELD_DATA_t FieldData[MAX_NUM_FIELDS];

	printf("Test FieldInfo\n");

// Test sull'handle
	test[numtest].testid = 20301;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "FieldInfo: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetFieldInfo(5, 1, &FieldData[0]);
	printf("Ritorno della FieldInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Test numero campi
	test[numtest].testid = 20302;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "FieldInfo: parametro NumFields");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetFieldInfo(handle, -1, &FieldData[0]);
	printf("Ritorno della FieldInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Test FieldData
	test[numtest].testid = 20303;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "FieldInfo: parametro FieldData");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetFieldInfo(handle, 2, NULL);
	printf("Ritorno della FieldInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Test campo
	NumFields = 1;
	strcpy(FieldData[0].Name, "PopOI");
	test[numtest].weight = 1;
	test[numtest].testid = 20304;
	strcpy(test[numtest].desc, "FieldInfo: nome campo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetFieldInfo(handle, NumFields, &FieldData[0]);
	printf("Ritorno della FieldInfo: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 20305;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "FieldInfo: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GetFieldInfo(handle, 0, &FieldData[0]);
	printf("Ritorno della FieldInfo: [%d]\n", ret);
	if (!strcmp(FieldData[3].Name, "Campo4"))
		test[numtest].passed = 1;
	numtest++;

}

void TestDBFHandle()
{
	int ret = 0;

	printf("Test DBFHandle\n");

// Verifica funzionalita'
	// Handle errato
	test[numtest].testid = 20401;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "DBFHandle: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  IsDBFHandle(100);
	printf("Ritorno della DBFHandle: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

	// Handle corretto
	test[numtest].testid = 20402;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "DBFHandle: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  IsDBFHandle(handle);
	printf("Ritorno della DBFHandle: [%d]\n", ret);
	if (ret == LPC_OK)
		test[numtest].passed = 1;
	numtest++;

}


void TestClose()
{
	int ret = 0;

	printf("Test CLOSE\n");

	// Test sull'handle
	test[numtest].testid = 20501;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "CLOSE: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  CloseDatabaseFile(9);
	printf("Ritorno della CloseDatabase: [%d]\n", ret);
	if (ret != LPC_OK)
		test[numtest].passed = 1;
	numtest++;

	// funzionalita'
	test[numtest].testid = 20502;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "CLOSE: Funzionalita'");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  CloseDatabaseFile(handle);
	printf("Ritorno della CloseDatabase: [%d]\n", ret);
	if (ret == LPC_OK)
		test[numtest].passed = 1;
	numtest++;



}


void TestMisto()
{
	int ret = 0, ok = 0, i, num;
	DBF_INFO_t DBFInfo;
	char n[30];

	printf("Test MISTO\n");

	// Apertura piu' files
	test[numtest].testid = 20601;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "MISTO: Apertura piu' files");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	// Tolgo tutti i lock
	for (i = 0; i < MAX_DBF_OPEN; i++)
	{
		sprintf(n, "Prova%d.$$$", i+1);
		remove(n);
	}
	// Apro tutto
	for (i = 0; i < MAX_DBF_OPEN;i++)
	{
		sprintf(DBFTest[i].Nome, "Prova%d", i+1);
		DBFTest[i].handle = OpenDatabaseFile(DBFTest[i].Nome);
		if (DBFTest[i].handle < 0)
			break;
	}

	// Test se non sono uscito prima dal loop
	if (i == MAX_DBF_OPEN)
	{
		// Questa apertura deve dare errore ben specifico
		sprintf(DBFTest[i].Nome, "Prova%d", i+1);
		DBFTest[i].handle = OpenDatabaseFile(DBFTest[i].Nome);
		if (DBFTest[i].handle == LPC_NO_SPACE)
			test[numtest].passed = 1;
	}
	numtest++;

	// Funzionalita' mista di tutte le funzioni
	test[numtest].testid = 20602;
	test[numtest].weight = 15;
	strcpy(test[numtest].desc, "MISTO: Is, Close, Open");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(num = MAX_DBF_OPEN;num > 0;)
	{
		ok = 0;
		i = rand() % MAX_DBF_OPEN;
		// Se l'handle  valido deve essere giusto il test altrimenti no
		if (DBFTest[i].handle >= 0)
		{
			num--;
			// Handle valido
			ret = IsDBFHandle(DBFTest[i].handle);
			if (ret != LPC_OK)
				break;
			// Info e nome
			ret =  GetDBFInfo(DBFTest[i].handle, &DBFInfo);
			if (ret != LPC_OK)
				break;
			sprintf(n, "Prova%d", i+1);
			if (strcmp(DBFInfo.Name, n))
				break;

			// Chiudo e a volte riapro
			ret = CloseDatabaseFile(DBFTest[i].handle);
			if (ret != LPC_OK)
				break;
			if (rand() % 2 == 1)
			{
				DBFTest[i].handle = OpenDatabaseFile(DBFTest[i].Nome);
				if (DBFTest[i].handle < 0)
					break;
				ret = CloseDatabaseFile(DBFTest[i].handle);
				if (ret != LPC_OK)
					break;
			}
			DBFTest[i].handle = -1;
			*DBFTest[i].Nome = '\0';
		}
		else
		{
			ret = IsDBFHandle(DBFTest[i].handle);
			if (ret == LPC_OK)
				break;
		}
		ok = 1;
	}
	if (ok)
		test[numtest].passed = 1;
	numtest++;


}


/*********************************
* RESULT
* Stampa i risultati. Se il parametro e' NULL allora resetta i valori
**********************************/
void Result(char *Func)
{
	int i, quanti, totval;

	// Svuoto il vettore
	if (!Func)
	{
		for (i = 0; i < MAX_TEST; i++)
			memset(&test[i], 0, sizeof(TEST_t));
		numtest = 0;
		return;
	}

	// Stampo gli ID
	printf("\n");
	printf("ID:   ");
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].testid);
	}

	// Stampo i pesi e prendo il valore totale
	printf("\n");
	printf("PESO: ");
	totval = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].weight);
		totval += test[i].weight;
	}

	// Stampo i risultati
	printf("\n");
	printf("PASS: ");
	quanti = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].passed);
		if (test[i].passed)
			quanti += test[i].weight;
	}

	printf("\n\nFine Test [%s]: Numtests:[%d] Totvalue:[%d] Passed:[%d] Score:[%3.2f]\n\n", Func, numtest, totval, quanti, (double) (quanti*100/((double)totval)));
}

/*! \} */	// Fine gruppo TestModulo1
/*! \} */	// Fine gruppo Modulo1

